// Copyright (c) 2023 mStar
// 
// Licensed under the EUPL, Version 1.2
// 
// You may not use this work except in compliance with the Licence.
// You should have received a copy of the Licence along with this work. If not, see:
// <https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12>.
// See the Licence for the specific language governing permissions and limitations under the Licence.
// 

pub mod diesel_db;

use anyhow::{Error, bail};
use diesel::{prelude::*, pg::{PgConnection, Pg}, sqlite::Sqlite};
use diesel_migrations::{EmbeddedMigrations, embed_migrations, MigrationHarness};

pub const POSTGRES_MIGRATIONS: EmbeddedMigrations = embed_migrations!("migrations/postgres");

pub enum DB {
    Postgres(PgConnection),
    Sqlite(SqliteConnection)
}

/// Try and connect to a database at the given url
/// The url can be a path to an sqlite db file or an url to a postgres db
pub fn establish_connection(db_url: &str) -> Result<DB, Error> {
    if db_url.starts_with("postgres") {
        Ok(DB::Postgres(establish_connection_postgres(db_url)?))
    } else {
        Ok(DB::Sqlite(establish_connection_sqlite(db_url)?))
    }
}

/// Try and connect to a postgres db
pub fn establish_connection_postgres(db_url: &str) -> Result<PgConnection, Error> {
    //SqliteConnection::establish(db_url).unwrap_or_else(|e| panic!("Error connecting to {}: {}", db_url, e))
    let mut conn = PgConnection::establish(db_url)
        .unwrap_or_else(|e| panic!("Error connecting to {}: {}", db_url, e));
    run_migrations_postgres(&mut conn)?;
    //c.run_pending_migrations(MIGRATIONS)?;
    Ok(conn)
}

fn run_migrations_postgres(conn: &mut impl MigrationHarness<Pg>) -> Result<(), Error> {
    match conn.run_pending_migrations(POSTGRES_MIGRATIONS) {
        Ok(_) => Ok(()),
        Err(e) => bail!("Migrations failed. Error {}", e)
    }
}

/// Try and connect to a sqlite db
pub fn establish_connection_sqlite(db_url: &str) -> Result<SqliteConnection, Error> {
    //SqliteConnection::establish(db_url).unwrap_or_else(|e| panic!("Error connecting to {}: {}", db_url, e))
    let mut conn = SqliteConnection::establish(db_url)
        .unwrap_or_else(|e| panic!("Error connecting to {}: {}", db_url, e));
    run_migrations_sqlite(&mut conn)?;
    //c.run_pending_migrations(MIGRATIONS)?;
    Ok(conn)
}

fn run_migrations_sqlite(conn: &mut impl MigrationHarness<Sqlite>) -> Result<(), Error> {
    match conn.run_pending_migrations(POSTGRES_MIGRATIONS) {
        Ok(_) => Ok(()),
        Err(e) => bail!("Migrations failed. Error {}", e)
    }
}